# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/that-ml-library',
                'doc_host': 'https://anhquan0412.github.io',
                'git_url': 'https://github.com/anhquan0412/that-ml-library',
                'lib_path': 'that_ml_library'},
  'syms': { 'that_ml_library.chart_plotting': { 'that_ml_library.chart_plotting._cramer_v': ( 'chart_plotting.html#_cramer_v',
                                                                                              'that_ml_library/chart_plotting.py'),
                                                'that_ml_library.chart_plotting.draw_sankey': ( 'chart_plotting.html#draw_sankey',
                                                                                                'that_ml_library/chart_plotting.py'),
                                                'that_ml_library.chart_plotting.get_cat_correlation': ( 'chart_plotting.html#get_cat_correlation',
                                                                                                        'that_ml_library/chart_plotting.py'),
                                                'that_ml_library.chart_plotting.get_correlation_by_threshold': ( 'chart_plotting.html#get_correlation_by_threshold',
                                                                                                                 'that_ml_library/chart_plotting.py'),
                                                'that_ml_library.chart_plotting.get_vif': ( 'chart_plotting.html#get_vif',
                                                                                            'that_ml_library/chart_plotting.py'),
                                                'that_ml_library.chart_plotting.params_2D_heatmap': ( 'chart_plotting.html#params_2d_heatmap',
                                                                                                      'that_ml_library/chart_plotting.py'),
                                                'that_ml_library.chart_plotting.params_3D_heatmap': ( 'chart_plotting.html#params_3d_heatmap',
                                                                                                      'that_ml_library/chart_plotting.py'),
                                                'that_ml_library.chart_plotting.pdp_categorical_only': ( 'chart_plotting.html#pdp_categorical_only',
                                                                                                         'that_ml_library/chart_plotting.py'),
                                                'that_ml_library.chart_plotting.pdp_numerical_only': ( 'chart_plotting.html#pdp_numerical_only',
                                                                                                       'that_ml_library/chart_plotting.py'),
                                                'that_ml_library.chart_plotting.plot_cat_correlation': ( 'chart_plotting.html#plot_cat_correlation',
                                                                                                         'that_ml_library/chart_plotting.py'),
                                                'that_ml_library.chart_plotting.plot_classification_tree_sklearn': ( 'chart_plotting.html#plot_classification_tree_sklearn',
                                                                                                                     'that_ml_library/chart_plotting.py'),
                                                'that_ml_library.chart_plotting.plot_confusion_matrix': ( 'chart_plotting.html#plot_confusion_matrix',
                                                                                                          'that_ml_library/chart_plotting.py'),
                                                'that_ml_library.chart_plotting.plot_feature_importances': ( 'chart_plotting.html#plot_feature_importances',
                                                                                                             'that_ml_library/chart_plotting.py'),
                                                'that_ml_library.chart_plotting.plot_ice_pair': ( 'chart_plotting.html#plot_ice_pair',
                                                                                                  'that_ml_library/chart_plotting.py'),
                                                'that_ml_library.chart_plotting.plot_learning_curve': ( 'chart_plotting.html#plot_learning_curve',
                                                                                                        'that_ml_library/chart_plotting.py'),
                                                'that_ml_library.chart_plotting.plot_permutation_importances': ( 'chart_plotting.html#plot_permutation_importances',
                                                                                                                 'that_ml_library/chart_plotting.py'),
                                                'that_ml_library.chart_plotting.plot_prediction_distribution': ( 'chart_plotting.html#plot_prediction_distribution',
                                                                                                                 'that_ml_library/chart_plotting.py'),
                                                'that_ml_library.chart_plotting.plot_residuals': ( 'chart_plotting.html#plot_residuals',
                                                                                                   'that_ml_library/chart_plotting.py'),
                                                'that_ml_library.chart_plotting.plot_tree_dtreeviz': ( 'chart_plotting.html#plot_tree_dtreeviz',
                                                                                                       'that_ml_library/chart_plotting.py'),
                                                'that_ml_library.chart_plotting.plot_validation_curve': ( 'chart_plotting.html#plot_validation_curve',
                                                                                                          'that_ml_library/chart_plotting.py')},
            'that_ml_library.data_preprocess': { 'that_ml_library.data_preprocess.one_hot_cat': ( 'data_preprocess.html#one_hot_cat',
                                                                                                  'that_ml_library/data_preprocess.py'),
                                                 'that_ml_library.data_preprocess.preprocessing_general': ( 'data_preprocess.html#preprocessing_general',
                                                                                                            'that_ml_library/data_preprocess.py'),
                                                 'that_ml_library.data_preprocess.process_missing_values': ( 'data_preprocess.html#process_missing_values',
                                                                                                             'that_ml_library/data_preprocess.py'),
                                                 'that_ml_library.data_preprocess.scale_num_cols': ( 'data_preprocess.html#scale_num_cols',
                                                                                                     'that_ml_library/data_preprocess.py')},
            'that_ml_library.ml_helpers': { 'that_ml_library.ml_helpers.do_param_search': ( 'ml_helpers.html#do_param_search',
                                                                                            'that_ml_library/ml_helpers.py'),
                                            'that_ml_library.ml_helpers.get_adaboost_info': ( 'ml_helpers.html#get_adaboost_info',
                                                                                              'that_ml_library/ml_helpers.py'),
                                            'that_ml_library.ml_helpers.run_logistic_regression': ( 'ml_helpers.html#run_logistic_regression',
                                                                                                    'that_ml_library/ml_helpers.py'),
                                            'that_ml_library.ml_helpers.run_multinomial_statmodel': ( 'ml_helpers.html#run_multinomial_statmodel',
                                                                                                      'that_ml_library/ml_helpers.py'),
                                            'that_ml_library.ml_helpers.run_sklearn_model': ( 'ml_helpers.html#run_sklearn_model',
                                                                                              'that_ml_library/ml_helpers.py'),
                                            'that_ml_library.ml_helpers.show_both_cv': ( 'ml_helpers.html#show_both_cv',
                                                                                         'that_ml_library/ml_helpers.py'),
                                            'that_ml_library.ml_helpers.summarize_cv_results': ( 'ml_helpers.html#summarize_cv_results',
                                                                                                 'that_ml_library/ml_helpers.py'),
                                            'that_ml_library.ml_helpers.summarize_default_cv': ( 'ml_helpers.html#summarize_default_cv',
                                                                                                 'that_ml_library/ml_helpers.py'),
                                            'that_ml_library.ml_helpers.tune_sklearn_model': ( 'ml_helpers.html#tune_sklearn_model',
                                                                                               'that_ml_library/ml_helpers.py')},
            'that_ml_library.utils': { 'that_ml_library.utils.create_dir': ('utils.html#create_dir', 'that_ml_library/utils.py'),
                                       'that_ml_library.utils.val2list': ('utils.html#val2list', 'that_ml_library/utils.py')}}}
